@@ -17,12 +17,14 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 import re
-import gtk
 import posixpath
 
 from gettext import gettext as _
 from string import Template
 
+from gi.repository import Gtk, Gdk, Pango, PangoCairo
+import cairo
+
 from checkbox.job import UNINITIATED
 from checkbox.user_interface import (UserInterface,
     NEXT, YES_ANSWER, NO_ANSWER, SKIP_ANSWER,
@@ -31,7 +33,6 @@ from checkbox.user_interface import (UserInterface,
 # Import to register HyperTextView type with gtk
 from checkbox_gtk.hyper_text_view import HyperTextView
 
-
 ANSWER_TO_BUTTON = {
     YES_ANSWER: "radio_button_yes",
     NO_ANSWER: "radio_button_no",
@@ -68,8 +69,8 @@ class GTKInterface(UserInterface):
         super(GTKInterface, self).__init__(title, data_path)
 
         # Load UI
-        gtk.window_set_default_icon_name("checkbox")
-        self.widgets = gtk.Builder()
+        Gtk.Window.set_default_icon_name("checkbox")
+        self.widgets = Gtk.Builder()
         self.widgets.set_translation_domain(self.gettext_domain)
         self.widgets.add_from_file(posixpath.join(data_path,
             "checkbox-gtk.ui"))
@@ -77,21 +78,38 @@ class GTKInterface(UserInterface):
 
         # Set background color for head
         eventbox_head = self._get_widget("eventbox_head")
-        eventbox_head.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
+        color = Gdk.color_parse("#F0EBE2")
+        #HACK: LP #839675
+        if isinstance(color, tuple): color = color[1]
+        eventbox_head.modify_bg(Gtk.StateType.NORMAL, color)
 
-        # Set dialog title
+        # Set and apply initial/default dialog title
+        self._app_title = title
         self._dialog = self._get_widget("dialog_main")
-        self._dialog.set_title(title)
+        self._dialog.set_title(self._app_title)
+
+        #Setup and handler to render pixmap and translatable description text.
+        self.IMAGE_HEAD_BACKGROUND = posixpath.join(data_path,
+            "checkbox-gtk-head.png")
+        self.FONT = "Ubuntu"
+        self.TEXT =_("System Testing")
+
+        image_head=self._get_widget("image_head")
+        try:
+            image_head.connect("draw",self.draw_image_head)
+        except TypeError:
+            #Looks like GTK+ 2.x
+            image_head.connect("expose-event",self.draw_image_head)
 
         # Set wait transient for dialog
         self._wait = self._get_widget("window_wait")
         self._wait.set_transient_for(self._dialog)
         self._wait.realize()
-        self._wait.window.set_functions(gtk.gdk.FUNC_MOVE)
+        self._wait.get_window().set_functions(Gdk.WMFunction.MOVE)
 
         # Set shorthand for notebook
         self._notebook = self._get_widget("notebook_main")
-        self._handler_id = None
+        self._handlers = {}
 
     def _get_widget(self, name):
         return self.widgets.get_object(name)
@@ -113,7 +131,7 @@ class GTKInterface(UserInterface):
     def _get_text_view(self, name):
         widget = self._get_widget(name)
         buffer = widget.get_buffer()
-        text = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
+        text = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter(), True)
         return text
 
     def _set_label(self, name, label=""):
@@ -125,7 +143,7 @@ class GTKInterface(UserInterface):
         widget.set_text(text)
 
     def _set_text_view(self, name, text=""):
-        buffer = gtk.TextBuffer()
+        buffer = Gtk.TextBuffer()
         buffer.set_text(text)
         widget = self._get_widget(name)
         widget.set_buffer(buffer)
@@ -135,9 +153,10 @@ class GTKInterface(UserInterface):
             self.show_url(anchor)
 
         widget = self._get_widget(name)
-        widget.connect("anchor-clicked", clicked)
+        if widget not in self._handlers:
+            self._handlers[widget] = widget.connect("anchor-clicked", clicked)
 
-        buffer = gtk.TextBuffer()
+        buffer = Gtk.TextBuffer()
         widget.set_buffer(buffer)
 
         url_regex = r"https?://[^\s]+"
@@ -188,21 +207,28 @@ class GTKInterface(UserInterface):
 
         return state
 
+    def _set_main_title(self, test_name=None):
+        title = self._app_title
+        if test_name:
+            title += " - %s" % test_name
+        self._get_widget("dialog_main").set_title(title)
+
     def _run_dialog(self, dialog=None):
         def on_dialog_response(dialog, response, self):
             # Keep dialog alive when the button that has been clicked
             # isn't meant to move to a previous/next window
             # To do this, please link the RESPONSE_REJECT code to such
             # button in the glade file
-            if response != gtk.RESPONSE_REJECT:
+            if response != Gtk.ResponseType.REJECT:
                 self.direction = response
-                gtk.main_quit()
+                Gtk.main_quit()
 
         dialog = dialog or self._dialog
         dialog.connect("response", on_dialog_response, self)
+        dialog.set_default_response(NEXT)
         dialog.show()
-        gtk.main()
-        if self.direction == gtk.RESPONSE_DELETE_EVENT:
+        Gtk.main()
+        if self.direction == Gtk.ResponseType.DELETE_EVENT:
             raise KeyboardInterrupt
 
     def show_progress_start(self, message):
@@ -220,11 +246,13 @@ class GTKInterface(UserInterface):
 
     def show_progress_pulse(self):
         self._get_widget("progressbar_wait").pulse()
-        while gtk.events_pending():
-            gtk.main_iteration()
+        while Gtk.events_pending():
+            Gtk.main_iteration()
 
     @GTKHack
     def show_text(self, text, previous=None, next=None):
+        #Reset window title
+        self._set_main_title()
         # Set buttons
         previous_state = self._set_button("button_previous", previous)
         next_state = self._set_button("button_next", next)
@@ -241,6 +269,8 @@ class GTKInterface(UserInterface):
 
     @GTKHack
     def show_entry(self, text, value, previous=None, next=None):
+        #Reset window title
+        self._set_main_title()
         # Set buttons
         previous_state = self._set_button("button_previous", previous)
         next_state = self._set_button("button_next", next)
@@ -272,7 +302,8 @@ class GTKInterface(UserInterface):
         vbox = self._get_widget("vbox_options_list")
         for option in options:
             label = "_%s%s" % (option[0].upper(), option[1:])
-            check_button = gtk.CheckButton(label)
+            check_button = Gtk.CheckButton(label = label,
+                                           use_underline = True)
             check_button.get_child().set_line_wrap(True)
             check_button.show()
             option_table[option] = check_button
@@ -318,7 +349,9 @@ class GTKInterface(UserInterface):
         vbox = self._get_widget("vbox_options_list")
         for option in options:
             label = "_%s%s" % (option[0].upper(), option[1:])
-            radio_button = gtk.RadioButton(option_group, label)
+            radio_button = Gtk.RadioButton(group = option_group,
+                                           label = label,
+                                           use_underline = True)
             radio_button.get_child().set_line_wrap(True)
             radio_button.show()
             option_table[option] = radio_button
@@ -343,6 +376,9 @@ class GTKInterface(UserInterface):
 
     @GTKHack
     def show_tree(self, text, options={}, default={}):
+        #Reset window title
+        self._set_main_title()
+
         (COLUMN_TEXT, COLUMN_ACTIVE) = range(2)
 
         # Set buttons
@@ -351,8 +387,9 @@ class GTKInterface(UserInterface):
         self._set_show("button_deselect_all", True)
 
         # Set options
-        treestore = gtk.TreeStore(str, bool)
-        treeview = gtk.TreeView(treestore)
+        treestore = Gtk.TreeStore(str, bool)
+        treeview = Gtk.TreeView()
+        treeview.set_model(treestore)
         treeview.set_headers_visible(False)
         treeview.show()
 
@@ -375,15 +412,22 @@ class GTKInterface(UserInterface):
                 active = not treestore.get_value(iter, COLUMN_ACTIVE)
             treestore.set_value(iter, COLUMN_ACTIVE, active)
 
-            # Set parents
-            def set_parents(iter):
-                parent = treestore.iter_parent(iter)
-                if parent:
-                    treestore.set_value(parent, COLUMN_ACTIVE, active)
-                    set_parents(parent)
+            # Set ancestors
+            def set_ancestors(iterator):
+                parent_row = treestore[iterator].parent
+                if active:
+                    while parent_row:
+                        parent_row[COLUMN_ACTIVE] = active
+                        parent_row = parent_row.parent
+                else:
+                    while parent_row:
+                        if any(child[COLUMN_ACTIVE]
+                               for child in parent_row.iterchildren()):
+                            break
+                        parent_row[COLUMN_ACTIVE] = active
+                        parent_row = parent_row.parent
 
-            if active:
-                set_parents(iter)
+            set_ancestors(iter)
 
             # Set children
             def set_children(iter):
@@ -396,15 +440,15 @@ class GTKInterface(UserInterface):
             set_children(iter)
 
         # Toggle column
-        cell = gtk.CellRendererToggle()
+        cell = Gtk.CellRendererToggle()
         cell.connect("toggled",
             lambda t, p: toggle_tree(treestore.get_iter_from_string(p)))
-        col = gtk.TreeViewColumn(None, cell, active=COLUMN_ACTIVE)
+        col = Gtk.TreeViewColumn(None, cell, active=COLUMN_ACTIVE)
         treeview.append_column(col)
 
         # Text column
-        cell = gtk.CellRendererText()
-        col = gtk.TreeViewColumn(None, cell, text=COLUMN_TEXT)
+        cell = Gtk.CellRendererText()
+        col = Gtk.TreeViewColumn(None, cell, text=COLUMN_TEXT)
         treeview.append_column(col)
 
         # Select and deselect buttons
@@ -419,6 +463,7 @@ class GTKInterface(UserInterface):
             button.connect("clicked", click_button,
                 "deselect" not in button_name)
 
+        self._set_hyper_text_view("hyper_text_view_options", text)
         self._run_dialog()
 
         # Unset buttons
@@ -459,9 +504,13 @@ class GTKInterface(UserInterface):
 
     @GTKHack
     def show_test(self, test, runner):
+        #Set window title to reflect current test
+        self._set_main_title(test["name"])
+
         self._set_sensitive("button_test", False)
         self._notebook.set_current_page(2)
 
+
         # Set test description
         if re.search(r"\$output", test["description"]):
             self._run_test(test, runner)
@@ -473,9 +522,9 @@ class GTKInterface(UserInterface):
         if "command" in test:
             self._set_sensitive("button_test", True)
             button_test = self._get_widget("button_test")
-            if self._handler_id:
-                button_test.disconnect(self._handler_id)
-            self._handler_id = button_test.connect("clicked",
+            if button_test in self._handlers:
+                button_test.disconnect(self._handlers[button_test])
+            self._handlers[button_test] = button_test.connect("clicked",
                 lambda w, t=test, r=runner: self._run_test(t, r))
 
         self._set_text_view("text_view_comment", test.get("data", ""))
@@ -491,15 +540,15 @@ class GTKInterface(UserInterface):
         test["status"] = self._get_radio_button(BUTTON_TO_STATUS)
 
     def show_info(self, text, options=[], default=None):
-        message_dialog = gtk.MessageDialog(parent=self._dialog,
-            type=gtk.MESSAGE_INFO,
-            buttons=gtk.BUTTONS_NONE,
+        message_dialog = Gtk.MessageDialog(parent=self._dialog,
+            type=Gtk.MessageType.INFO,
+            buttons=Gtk.ButtonsType.NONE,
             message_format=text)
         message_dialog.set_modal(True)
         message_dialog.set_title(_("Info"))
 
         for index, option in enumerate(options):
-            button = getattr(gtk, "STOCK_%s" % option.upper())
+            button = getattr(Gtk, "STOCK_%s" % option.upper())
             message_dialog.add_buttons(button, index)
 
         self._run_dialog(message_dialog)
@@ -510,13 +559,38 @@ class GTKInterface(UserInterface):
         return options[response]
 
     def show_error(self, text):
-        message_dialog = gtk.MessageDialog(parent=self._dialog,
-            type=gtk.MESSAGE_ERROR,
-            buttons=gtk.BUTTONS_NONE,
+        message_dialog = Gtk.MessageDialog(parent=self._dialog,
+            type=Gtk.MessageType.ERROR,
+            buttons=Gtk.ButtonsType.NONE,
             message_format=text)
         message_dialog.set_modal(True)
         message_dialog.set_title(_("Error"))
-        message_dialog.set_default_response(1)
-        message_dialog.add_buttons(gtk.STOCK_CLOSE, NEXT)
+        message_dialog.set_default_response(NEXT)
+        message_dialog.add_buttons(Gtk.STOCK_CLOSE, NEXT)
         self._run_dialog(message_dialog)
         message_dialog.hide()
+
+    def draw_image_head(self, widget, data):
+        #Render the background we read from an image
+        background_image = cairo.ImageSurface.create_from_png(
+                self.IMAGE_HEAD_BACKGROUND)
+        cairo_drawing_context = Gdk.cairo_create(widget.get_window())
+        cairo_drawing_context.set_source_surface(background_image,0,0)
+        cairo_drawing_context.paint()
+        #Text rendering with Pango is more involved but potentially
+        #better for l10n and i18n purposes
+        font_description = Pango.FontDescription()
+        font_description.set_family(self.FONT)
+        font_description.set_weight(Pango.Weight.NORMAL)
+        font_description.set_absolute_size(12 * Pango.SCALE)
+
+        pango_drawing_context = widget.get_pango_context()
+        pango_text_layout = Pango.Layout(context=pango_drawing_context)
+        pango_text_layout.set_font_description(font_description)
+        pango_text_layout.set_text(self.TEXT, -1)
+
+        #Color to render text
+        cairo_drawing_context.set_source_rgb(0,0,0)
+        #Position to render text
+        cairo_drawing_context.move_to(75,45)
+        PangoCairo.show_layout(cairo_drawing_context,pango_text_layout)
