@@ -55,7 +55,9 @@ class JobsPrompt(Plugin):
         self._store = None
 
         for (rt, rh) in [
+             ("expose-msgstore", self.expose_msgstore),
              ("begin-persist", self.begin_persist),
+             ("begin-recover", self.begin_recover),
              ("ignore-jobs", self.ignore_jobs),
              ("prompt-job", self.prompt_job),
              ("prompt-jobs", self.prompt_jobs),
@@ -64,9 +66,16 @@ class JobsPrompt(Plugin):
              ("report-job", self.report_job)]:
             self._manager.reactor.call_on(rt, rh)
 
+    def expose_msgstore(self):
+        self._manager.reactor.fire("store-access", self.store)
+
     def begin_persist(self, persist):
         self._persist = persist
 
+    def begin_recover(self, recover):
+        if not recover:
+            self.store.delete_all_messages()
+
     def ignore_jobs(self, jobs):
         self._ignore = jobs
 
